@@ -1,3 +1,4 @@
+import os
 from datetime import datetime
 from time import sleep
 from typing import Any, Iterable, List, Optional, Type
@@ -26,7 +27,22 @@ class K8sClients:
         # pylint: disable=import-outside-toplevel
         from kubernetes import client, config
 
-        config.load_kube_config()
+        if os.path.exists(
+            os.path.expanduser(config.kube_config.KUBE_CONFIG_DEFAULT_LOCATION)
+        ):
+            # found the kubeconfig file, let's load it!
+            config.load_kube_config()
+        elif (
+            config.incluster_config.SERVICE_HOST_ENV_NAME in os.environ
+            and config.incluster_config.SERVICE_PORT_ENV_NAME in os.environ
+        ):
+            # We are running inside a cluster, let's load the in-cluster configuration.
+            config.load_incluster_config()
+        else:
+            raise exceptions.TutorError(
+                f"there is no Kubernetes configuration file located in {config.kube_config.KUBE_CONFIG_DEFAULT_LOCATION}, and the service environment variables {config.incluster_config.SERVICE_HOST_ENV_NAME} and {config.incluster_config.SERVICE_PORT_ENV_NAME} have not been defined. To connect to a cluster, please configure your host appropriately."
+            )
+
         self._batch_api = None
         self._core_api = None
         self._client = client
@@ -280,18 +296,11 @@ def start(context: K8sContext, names: List[str]) -> None:
     names = names or ["all"]
     for name in names:
         if name == "all":
-            # Create volumes
-            kubectl_apply(
-                context.root,
-                "--wait",
-                "--selector",
-                "app.kubernetes.io/component=volume",
-            )
-            # Create everything else except jobs
+            # Create everything except jobs
             kubectl_apply(
                 context.root,
                 "--selector",
-                "app.kubernetes.io/component notin (job,volume,namespace)",
+                "app.kubernetes.io/component notin (job,namespace)",
             )
         else:
             kubectl_apply(
