@@ -6,16 +6,26 @@ import shutil
 import typing as t
 from copy import deepcopy
 
+import importlib_resources
 import jinja2
-import pkg_resources
 
 from tutor import exceptions, fmt, hooks, plugins, utils
 from tutor.__about__ import __app__, __version__
 from tutor.types import Config, ConfigValue
 
-TEMPLATES_ROOT = pkg_resources.resource_filename("tutor", "templates")
+TEMPLATES_ROOT = str(importlib_resources.files("tutor") / "templates")
 VERSION_FILENAME = "version"
-BIN_FILE_EXTENSIONS = [".ico", ".jpg", ".patch", ".png", ".ttf", ".woff", ".woff2"]
+BIN_FILE_EXTENSIONS = [
+    ".ico",
+    ".jpg",
+    ".otf",
+    ".patch",
+    ".png",
+    ".ttf",
+    ".webp",
+    ".woff",
+    ".woff2",
+]
 JinjaFilter = t.Callable[..., t.Any]
 
 
@@ -54,6 +64,7 @@ def _prepare_environment() -> None:
             ("HOST_USER_ID", utils.get_user_id()),
             ("TUTOR_APP", __app__.replace("-", "_")),
             ("TUTOR_VERSION", __version__),
+            ("is_docker_rootless", utils.is_docker_rootless),
         ],
     )
 
@@ -161,7 +172,7 @@ class Renderer:
             try:
                 patches.append(self.render_str(patch))
             except exceptions.TutorError:
-                fmt.echo_error(f"Error rendering patch '{name}': {patch}")
+                fmt.echo_error(f"Error rendering patch '{name}':\n{patch}")
                 raise
         rendered = separator.join(patches)
         if rendered:
@@ -169,7 +180,10 @@ class Renderer:
         return rendered
 
     def render_str(self, text: str) -> str:
-        template = self.environment.from_string(text)
+        try:
+            template = self.environment.from_string(text)
+        except jinja2.exceptions.TemplateSyntaxError as e:
+            raise exceptions.TutorError(f"Template syntax error: {e.args[0]}")
         return self.__render(template)
 
     def render_template(self, template_name: str) -> t.Union[str, bytes]:
@@ -180,8 +194,7 @@ class Renderer:
         The template_name *always* uses "/" separators, and is not os-dependent. Do not pass the result of
         os.path.join(...) to this function.
         """
-        if is_binary_file(template_name):
-            # Don't try to render binary files
+        if not hooks.Filters.IS_FILE_RENDERED.apply(True, template_name):
             return self.environment.read_bytes(template_name)
 
         try:
@@ -372,6 +385,8 @@ def render_unknown(config: Config, value: t.Any) -> t.Any:
         return render_str(config, value)
     if isinstance(value, dict):
         return {k: render_unknown(config, v) for k, v in value.items()}
+    if isinstance(value, list):
+        return [render_unknown(config, v) for v in value]
     return value
 
 
@@ -450,6 +465,9 @@ def get_release(version: str) -> str:
         "13": "maple",
         "14": "nutmeg",
         "15": "olive",
+        "16": "palm",
+        "17": "quince",
+        "18": "redwood",
     }[version.split(".", maxsplit=1)[0]]
 
 
@@ -521,7 +539,7 @@ def _delete_plugin_templates(plugin: str, root: str, _config: Config) -> None:
     Delete plugin env files on unload.
     """
     targets = hooks.Filters.ENV_TEMPLATE_TARGETS.iterate_from_context(
-        hooks.Contexts.APP(plugin).name
+        hooks.Contexts.app(plugin).name
     )
     for src, dst in targets:
         path = pathjoin(root, dst.replace("/", os.sep), src.replace("/", os.sep))
@@ -533,3 +551,24 @@ def _delete_plugin_templates(plugin: str, root: str, _config: Config) -> None:
                 raise exceptions.TutorError(
                     f"Could not delete file {e.filename} from plugin {plugin} in folder {path}"
                 )
+
+
+@hooks.Filters.IS_FILE_RENDERED.add()
+def _do_not_render_binary_files(result: bool, path: str) -> bool:
+    """
+    Determine whether a file should be rendered based on its binary nature.
+
+    This function checks if a file is binary and updates the rendering decision accordingly.
+    If the initial decision (`result`) is to render the file, but the file is detected as binary,
+    the function will override this decision to prevent rendering.
+
+    Parameters:
+    - result (bool): The initial decision on whether the file should be rendered.
+    - path (str): The file path to check for binary content.
+
+    Returns:
+    - bool: False if the file is binary and was initially set to be rendered, otherwise returns the initial decision.
+    """
+    if result and is_binary_file(path):
+        result = False
+    return result
