@@ -1,17 +1,13 @@
 from tutor import fmt, hooks, utils
-from tutor.types import Config, get_typed
-
-
-def get_tag(config: Config, name: str) -> str:
-    key = "DOCKER_IMAGE_" + name.upper().replace("-", "_")
-    return get_typed(config, key, str)
 
 
 def build(path: str, tag: str, *args: str) -> None:
     fmt.echo_info(f"Building image {tag}")
-    command = hooks.Filters.DOCKER_BUILD_COMMAND.apply(
-        ["build", "-t", tag, *args, path]
-    )
+    build_command = ["build", f"--tag={tag}", *args, path]
+    # `buildx` can be removed once Tutor requires Docker v23+. At that point, BuildKit will be
+    # enabled by default for all Docker users.
+    build_command.insert(0, "buildx")
+    command = hooks.Filters.DOCKER_BUILD_COMMAND.apply(build_command)
     utils.docker(*command)
 
 
