@@ -4,7 +4,7 @@
 LOGIN_REDIRECT_WHITELIST = ["{{ CMS_HOST }}"]
 
 # Better layout of honor code/tos links during registration
-REGISTRATION_EXTRA_FIELDS["terms_of_service"] = "required"
+REGISTRATION_EXTRA_FIELDS["terms_of_service"] = "hidden"
 REGISTRATION_EXTRA_FIELDS["honor_code"] = "hidden"
 
 # Fix media files paths
@@ -36,14 +36,9 @@ CACHES["staticfiles"] = {
     "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
     "LOCATION": "staticfiles_lms",
 }
-CACHES["ora2-storage"] = {
-    "KEY_PREFIX": "ora2-storage",
-    "BACKEND": "django_redis.cache.RedisCache",
-    "LOCATION": "redis://{% if REDIS_USERNAME and REDIS_PASSWORD %}{{ REDIS_USERNAME }}:{{ REDIS_PASSWORD }}{% endif %}@{{ REDIS_HOST }}:{{ REDIS_PORT }}/{{ OPENEDX_CACHE_REDIS_DB }}",
-}
 
 # Create folders if necessary
-for folder in [DATA_DIR, LOG_DIR, MEDIA_ROOT, STATIC_ROOT_BASE, ORA2_FILEUPLOAD_ROOT]:
+for folder in [DATA_DIR, LOG_DIR, MEDIA_ROOT, STATIC_ROOT, ORA2_FILEUPLOAD_ROOT]:
     if not os.path.exists(folder):
         os.makedirs(folder, exist_ok=True)
 
